"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const LinkedQueue_js_1 = require("./LinkedQueue.js");
class AsyncQueue {
    queue = new LinkedQueue_js_1.LinkedQueue();
    pushTx = undefined;
    pushRx = new Promise((resolve) => (this.pushTx = resolve));
    shiftTx = undefined;
    shiftRx = new Promise((resolve) => (this.shiftTx = resolve));
    isDone = false;
    constructor(iterable) {
        if (iterable) {
            this.queue.push(...iterable);
        }
    }
    async *[Symbol.asyncIterator]() {
        while (true) {
            if (this.queue.size) {
                yield this.shift();
            }
            else if (this.isDone) {
                break;
            }
            else {
                await this.pushRx;
            }
        }
    }
    shift() {
        const value = this.queue.shift();
        this.shiftTx?.();
        this.shiftRx = new Promise((tx) => (this.shiftTx = tx));
        return value;
    }
    push(...values) {
        this.queue.push(...values);
        this.pushTx?.();
        this.pushRx = new Promise((tx) => (this.pushTx = tx));
    }
    done() {
        this.isDone = true;
        this.pushTx?.();
    }
    get size() {
        return this.queue.size;
    }
    async backpressure(max) {
        while (this.size >= max) {
            await this.shiftRx;
        }
    }
}
exports.default = AsyncQueue;
//# sourceMappingURL=AsyncQueue.js.map